/**
 *	Copyright (C) 2011-2015 Docuteam GmbH
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 3
 *	as published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.docuteam.tools.os;

import java.io.File;


public abstract class OperatingSystem
{
	//	===========================================================================================
	//	========	Structure				=======================================================
	//	===========================================================================================

	//	========	Static Final Public		=======================================================

	//	========	Static Final Private	=======================================================

	//	========	Static Public			=======================================================

	static private String			OSName;
	static private String			OSVersion;
	static private String			UserName;
	static private String			UserLanguage;
	static private String			UserHome;
	static private String			JavaTempDir;

	static private Boolean			IsWindows = false;
	static private Boolean			IsMacOSX = false;
	static private Boolean			IsLinux = false;

	//	========	Static Private			=======================================================

	//	===========================================================================================
	//	========	Main					=======================================================
	//	===========================================================================================

	//	===========================================================================================
	//	========	Methods					=======================================================
	//	===========================================================================================

	//	========	Static Initializer		=======================================================

	//	========	Static Public			=======================================================

	static public String osName()
	{
		initializeIfNecessary();

		return OSName;
	}

	static public String osVersion()
	{
		initializeIfNecessary();

		return OSVersion;
	}

	static public String userName()
	{
		initializeIfNecessary();

		return UserName;
	}

	static public String userLanguage()
	{
		initializeIfNecessary();

		return UserLanguage;
	}


	/**
	 *
	 * @return the user home dir INCLUDING trailing file separator.
	 */
	static public String userHome()
	{
		initializeIfNecessary();

		return UserHome;
	}

	/**
	 *
	 * @return the java temp dir INCLUDING trailing file separator.
	 */
	static public String javaTempDir()
	{
		initializeIfNecessary();

		return JavaTempDir;
	}


	static public boolean isWindows()
	{
		initializeIfNecessary();

		return IsWindows;
	}

	static public boolean isMacOSX()
	{
		initializeIfNecessary();

		return IsMacOSX;
	}

	static public boolean isLinux()
	{
		initializeIfNecessary();

		return IsLinux;
	}


	static public void beep(Integer times)
	{
		beep(times, 500);
	}

	static public void beep(Integer times, Integer mSexGap)
	{
		try
		{
			for (int i = 0; i < times; i++)
			{
				beep();
				Thread.sleep(mSexGap);
			}
		}
		catch (InterruptedException ex){}
	}

	static public void beep()
	{
		java.awt.Toolkit.getDefaultToolkit().beep();
	}

	//	========	Static Private			=======================================================

	static private void initializeIfNecessary()
	{
		if (OSName == null)		initialize();
	}


	static private void initialize()
	{
		OSName = System.getProperty("os.name");
		OSVersion = System.getProperty("os.version");
		UserName = System.getProperty("user.name");
		UserLanguage = System.getProperty("user.language");
		UserHome = System.getProperty("user.home");
		if (!UserHome.endsWith(File.separator))			UserHome += File.separator;
		JavaTempDir = System.getProperty("java.io.tmpdir");
		if (!JavaTempDir.endsWith(File.separator))		JavaTempDir += File.separator;

		String osNameLower = OSName.toLowerCase();
		if (osNameLower.contains("windows"))		IsWindows = true;
		else if (osNameLower.contains("os x"))		IsMacOSX = true;
		else if (osNameLower.contains("linux"))		IsLinux = true;
	}

	//	===========================================================================================
	//	========	Inner Classes			=======================================================
	//	===========================================================================================

}
